import { Middleware, Route } from '../base/http';
import { Logger, Message } from '../base/logger';
import { ServerProperties } from './ServerProperties';
/** @sealed */
declare class Server {
    private instance;
    private readonly port;
    private readonly logger;
    private i18n;
    static readonly i18nDefaultPath = "../resources/i18n.json";
    static readonly defaultMiddlewaresPath = "../middlewares";
    static readonly defaultRoutesPath = "../routes";
    private readonly httpHandlers;
    private initialized;
    private readonly i18nPath?;
    private readonly middlewaresPath?;
    private readonly routesPath?;
    constructor(properties: ServerProperties);
    init(): Promise<Server>;
    private postInit;
    private processHttpHandlers;
    addMiddleware(middleware: Middleware): Server;
    addRoute(route: Route): Server;
    logInfo(message: string): void;
    logError(message: string): void;
    logWarn(message: string): void;
    log(message: Message): void;
    private get;
    private post;
    registerRoutes(dir: string): Promise<Server>;
    registerMiddlewares(dir: string): Promise<Server>;
    getLogger(): Logger;
    i18nLoad(path: string): Server;
    start(callback?: () => any): void;
}
export { Server };
